<?php
/**
 * Plugin Messages (addon for plugin Adm_redacteurs)
 * Version  : 1.1 (2017/10/23)
 * Compatibility : Guppy v5.0+ et Adm_redacteurs v6.8+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

require_once ADMR_HELPERS_PATH.'/manage_records_helper.php';

class Manage_messages_helper extends Manage_records_helper
{
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    protected $CI; // object global
    protected $_lang = array();          // Données de langue
    protected $_config = array();        // Données de configuration
    protected $_params = array();        // Données récupérés par post ou get
    protected $_sort_options = array();	 // Options de tri

    protected $_doc_infos = array('name' => 'messages', 'type' => 'messages'); // Tableau contenant les informations du document (nom, type)
    protected $_writer_rights = array(); // Droits du rédacteur
    //
    
    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */    
    public function __construct($writer_rights)
    {
        $this -> _writer_rights = $this -> _set_writer_rights($writer_rights);

        parent :: __construct();
        
		$this -> CI -> load_model('writer_model');
        $this -> CI -> load_addon_model('messages', 'writer_messages_model', array('id' => $this -> _writer_rights['id'])); // Model des documents de guppy
        $this -> CI -> load_addon_helper('messages', 'plugin_menu_for_messages_helper');
        $this -> CI -> load_addon_helper('messages', 'messages_infos_helper');

        $this -> CI -> load_addon_lang('messages');
        $this -> CI -> load_addon_config('messages');        

        $this -> _lang = $this -> CI -> get_lang();
        $this -> _config = $this -> CI -> get_config();        

        /**
         * Controle de sécurité
         */
        if (! ADMR_IS_ADMIN && ! $this -> _is_doc_allowed()) {
            Url_helper :: redirect('display_error&type=disallowed');
        }
        //
    }

    /*
    |--------------------------------------------------------------------------
    | Methods
    |--------------------------------------------------------------------------
    */

	/**
	 * définit les options de tri des données
     * 
     * @access protected
     * @param mixed Options de tri
	 * @return void
	 */	
    protected function _set_writer_rights($writer_rights)
    {
        settype($writer_rights, 'array');
        
        return $this -> _writer_rights = $writer_rights;	// Droits du rédacteur         
    }

	// --------------------------------------------------------------------

	/**
	 * définit les options de tri des données
     * 
     * @access protected
     * @param mixed Options de tri
	 * @return void
	 */	
    protected function _set_sort_options()
    {
        return $this -> _sort_options = array('id_asc', 'id_desc', 'from_asc', 'from_desc', 'to_asc', 'to_desc', 'time_asc', 'time_desc', 'mode_asc', 'mode_desc');	// Options de tri         
    }

	// --------------------------------------------------------------------

	/**
	 * définit les variables utilisées dans la page 
     * 
     * @access protected
	 * @return void
	 */	
    protected function _set_params()
    {
        $this -> _params = array_merge(
            parent :: _set_params(),
            array(
                'docid'  => trim(Var_helper :: import('docid', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
                'msg_received_ids'  => trim(Var_helper :: import('msg_received_ids', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
                'msg_sent_ids'  => trim(Var_helper :: import('msg_sent_ids', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
                'author_ids'  => trim(Var_helper :: import('author_ids', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
                'receiver_ids'  => trim(Var_helper :: import('receiver_ids', '', NULL, array('trim'), array(array('Var_helper', 'is_valid_ids'))), '-'),
            )
        );
       
        return $this -> _params;
    }

	// --------------------------------------------------------------------

	/**
	 * gére les enregistrements
     * 
     * @access public
	 * @return string code html
	 */	
    public function manage()
    {        
        parent :: manage();
    }

	// --------------------------------------------------------------------

	/**
	 * recherche des enregistrements et redirige vers la page qui va afficher les résultats de la recherche
     * 
     * @access public
	 * @return void
	 */
    public function search()
    {
        $data = array(
            'lang' => $this -> _lang,
            'content' => $this -> _build_form_data_filtering(),
        );        
        
        $this -> render_page('public_search_records_layout', $data);       
    }

	// --------------------------------------------------------------------

    /**
     * Définit et renvoie le titre de la page 
     *
     * @access public
     * @return string titre de la page
     */        
    public function set_title_page()
    {
        return ucfirst($this -> _lang['manage_messages']);
    }
 
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie les ajouts (js et css) dans l'entête de la page
     * 
     * @access public
	 * @return string Chaine html contenant les ajouts js et css
	 */	
    public function set_header_addons()
    {
        $parent_addons = parent :: set_header_addons();
        
        $js = '';
        $js .= '<script type="text/javascript" src="'.MESSAGES_JS_PATH.'/search_messages.js"></script>'."\n";

        $css = '<link type="text/css" rel="stylesheet" href="'.MESSAGES_CSS_PATH.'/manage_messages.css" />';

        return $parent_addons.$css.$js;        
    }
   
	// --------------------------------------------------------------------

	/**
	 * définit et renvoie la description de la page ainsi que les recommendations
     * 
     * @access public
     * @return string description de la page
	 */	
    public function set_infos_page_content()
    {		
        return $this -> _lang['infos_page_manage_messages'];
    }
    
	// --------------------------------------------------------------------

    /**
     * Définit le contenu du menu qui est affiché dans le bandeau en haut de la page
     * 
     * @access public
     * @return string menu de la page
	 */	
    public function set_menu_plugin_items()
    {        
        if (ADMR_IS_ADMIN) {
            return Plugin_menu_for_messages_helper :: set_admin_items('messages', $this -> _lang, $this -> _config, $this -> _writer_rights);
        } else {
            return Plugin_menu_for_messages_helper :: set_public_items('messages', $this -> _lang, $this -> _config, $this -> _writer_rights);
        }
    }

	// --------------------------------------------------------------------

    /**
     * Définit le contenu des informations du plugin qui sont affichées en bas de la page
     *
     * @return array Tableau contenant les informations du plugin sous la forme array('version' => , 'licence' => )
     */
    public function set_infos_plugin()
    {
        return Messages_infos_helper :: set_plugin_infos($this -> _lang);
    }

	// --------------------------------------------------------------------

	/**
	 * synchronize les données des documents enregistrées pour le rédacteur avec ceux enregistrés dans le plugin messages
     * 
     * @access protected
     * @param mixed Options de tri
	 * @return void
	 */	
    public function synchronize()
    {
		Url_helper :: redirect('manage_messages');
    }

	// --------------------------------------------------------------------

	/**
	 * retourne les enregistrements
     * 
     * @access protected
	 * @return array
	 */
    protected function _get_records()
    {
        $doc_ids = empty($this -> _params['docid']) ? array() : explode('-', $this -> _params['docid']);        

        // Messages reçus uniquement
//        $records = $this -> CI -> writer_messages_model -> get(array('receiver_id' => $this -> _writer_rights['id'], 'id' => $doc_ids), $this -> _params['sort']);
        // Messages envoyé uniquement
//        $records = $this -> CI -> writer_messages_model -> get(array('author_id' => $this -> _writer_rights['id'], 'id' => $doc_ids), $this -> _params['sort']);

        // Tous les messages
        $records = $this -> CI -> writer_messages_model -> get(array('id' => $doc_ids), $this -> _params['sort']);
        
        return $records;
    }

	// --------------------------------------------------------------------

	/**
	 * retourne les enregistrements
     * 
     * @access protected
	 * @return array
	 */
/*    protected function _filter_records($records)
    {
        return $records;       
    }
*/    
	// --------------------------------------------------------------------

	// --------------------------------------------------------------------

	/**
	 * définit les options du menu de sélection des actions pour la mise à jours des enregistrements sous la forme 
     * array(
     *    array('label' => '', 'value' => '', 'attrs' => array()),
     * )
     * 
     * @access protected
	 * @return array
	 */	
    protected function _set_select_actions_for_update_records()
    {
        $actions = array();

        // Suppression des enregistrements
        if (!empty($this -> _writer_rights['messages_del'])) {
            $actions[] = array('label' => ucfirst($this -> _lang['delete_docs_selected']), 'value' => 'd', 'attrs' => array());
        }

        return $actions;
    }

	// --------------------------------------------------------------------

	/**
	 * Met à jour les données des enregistrements selon l'action sélectionnée
     * 
     * @access protected
     * @param array Identifiants des enregistrements à mettre à jour
     * @param string Identifiant de l'action à appliquer pour la mise à jour
	 * @return boolean
	 */
    protected function _update_records_by_action($records_ids, $action)
    {
        // Suppression des enregistrements
        if ($action == 'd') { 
            return $this -> _delete_records($records_ids);
        }        
        //           
        
        return FALSE;
    }
    
	// --------------------------------------------------------------------

	/**
	 * supprime les enregistrements sélectionnés
     * 
     * @access public
     * @param array Tableau contenant les identifiants des enregistrements
	 * @return boolean
	 */
    protected function _delete_records($records_ids)
    {      
        $this -> CI -> load_addon_model('messages', 'messages_model');

        if (!empty($this -> _writer_rights['messages_del'])) {
            $this -> CI -> writer_messages_model -> delete_by_id($records_ids);
              
            foreach($records_ids as $record_id) {
                $message = $this -> CI -> messages_model -> read($record_id);
                
                if ($message['author_id'] != $this -> _writer_rights['id']) {
                    $this -> CI -> writer_messages_model -> load_data($message['author_id']);
                } else {
                    $this -> CI -> writer_messages_model -> load_data($message['receiver_id']);
                }
                
                $record = $this -> CI -> writer_messages_model -> get(array('id' => $record_id));
                
                if (empty($record)) {
                    $this -> CI -> messages_model -> delete_by_id($record_id);        
                }
            } 
            
            $this -> CI -> writer_messages_model -> load_data($this -> _writer_rights['id']);
            
            return TRUE; 
        }
        
        return FALSE;
    }
    
	// --------------------------------------------------------------------

    /*
     * Affiche ou renvoie le contenu de la vue
     *
	 * @access protected
     * @param array Données des enregistrements
     * @param boolean Renvoie le contenu (TRUE) ou affiche (FALSE) de la vue
     * @return string|void
     */
    protected function _build_view($records)
    {
        return parent :: _build_view($records);
    }    

	// --------------------------------------------------------------------

    /**
	 * Définit et retourne le layout de la vue
     * 
	 * @access protected
     * @return string nom du fichier qui définit le layout
	 */	
    protected function _get_layout_for_manage_records_view()
    {
        return parent :: _get_layout_for_manage_records_view();   
    }

	// --------------------------------------------------------------------

	/**
	 * construit le formulaire de sélection des données
     * 
     * @access protected
	 * @return string Formulaire html
	 */	
    protected function _build_form_data_filtering()
    {
        return '';
    }

	// --------------------------------------------------------------------

	/**
	 * construit le tableau de données
     * 
	 * @return string Tableau HTML contenant les enregistrements
	 * @access public
	 */	
    protected function _build_data_table($records)
    {		
        return parent :: _build_data_table($records);        
    }

	// --------------------------------------------------------------------
  
	/**
	 * ajoute des données supplémentaires à l'enregistrement
     * 
	 * @access protected
     * @param array Tableau de données initiales de l'enregistrement
	 * @return array Tableau contenant les nouvelles données de l'enregistrement
	 */    
/*    protected function _add_more_data_for_record($record)
    {
        return $this -> CI -> messages_model -> get_by_id($record['id']);        
    }*/
    
	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie les entêtes de la table de données
     * 
	 * @access protected
	 * @return array Tableau contenant les entêtes du tableau de données
	 */    
    protected function _set_data_table_headers()
    {
		$headers = array(
            parent :: _set_data_table_header_for_id(),
            array('contents' => $this -> _anchor_sort_records(ucfirst($this -> _lang['author']), 'from'), 'attrs' => array('id' => 'rfrom', 'align'=> 'left', 'style' => 'width:15%')),
            array('contents' => $this -> _anchor_sort_records(ucfirst($this -> _lang['receiver']), 'to'), 'attrs' => array('id' => 'rto', 'align'=> 'left', 'style' => 'width:15%')),
			parent :: _set_data_table_header_for_description($this -> _lang['title']),
            array('contents' => $this -> _anchor_sort_records(ucfirst($this -> _lang['date']), 'time'), 'attrs' => array('id' => 'rtime', 'align'=> 'center', 'style' => 'width:15%')),
            array('contents' => $this -> _anchor_sort_records(ucfirst($this -> _lang['state']), 'mode'), 'attrs' => array('id' => 'rmode', 'align'=> 'center', 'style' => 'width:15%')),
            parent :: _set_data_table_header_for_commands(),
		);

        return $headers;
    }

	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie le contenu des cellules de la table de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return array Tableau contenant le contenu des cellules de la table de données
	 */    
    protected function _set_data_table_cells($record)
    {
    	$cells = array(
            parent :: _set_data_table_cell_for_id($record),
            array('contents' => $this -> _add_contents_for_td_author($record), 'attrs' => array('align' => 'left')),
            array('contents' => $this -> _add_contents_for_td_receiver($record), 'attrs' => array('align' => 'left')),
            parent :: _set_data_table_cell_for_description($record),
            array('contents' => $this -> _add_contents_for_td_date($record), 'attrs' => array('align' => 'center')),
            array('contents' => $this -> _add_contents_for_td_state($record), 'attrs' => array('align' => 'center')),
            parent :: _set_data_table_cell_for_commands($record),
    	);
        
        return $cells;
    }

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Id" du tableau de données
     * 
	 * @access private
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_id($record)
	{        
		return parent :: _add_contents_for_td_id($record);
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Auteur" du tableau de données
     * 
	 * @access private
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_author($record)
	{        
        $writer = $this -> CI -> writer_model -> get_by_id($record['author_id']);    

        return !empty($writer['name']) ? $writer['name'] : $this -> _get_admin_name();
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Destinataire" du tableau de données
     * 
	 * @access private
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_receiver($record)
	{        
        $writer = $this -> CI -> writer_model -> get_by_id($record['receiver_id']);
        
        return !empty($writer['name']) ? $writer['name'] : $this -> _get_admin_name();
	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie le nom de l'administrateur
     * 
	 * @access private
	 * @return string Nom de l'administrateur
	 */
	private function _get_admin_name()
	{        
        return '<em class="admin_name">'.$this-> _config['user'][0].'</em>';
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Date" du tableau de données
     * 
	 * @access private
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_date($record)
	{        
        return date('d/m/Y H:m', $record['create_datetime']);
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Etat" du tableau de données
     * 
	 * @access private
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_state($record)
	{        
		$mode = $record['mode'];
        
        if ($mode == 1) { // non lu
            $view = '&nbsp;'.Html_helper :: addImg(
    			$path = ADMR_IMG_NEW,
    			$alt = ucfirst($this -> _lang['new']),
    			$attr = array('title' => $alt, 'border' => '0')
    		);		  
		} else { // lu
            $view = '&nbsp;'.Html_helper :: addImg(
    			$path = ADMR_IMG_APPROVED,
    			$alt = ucfirst($this -> _lang['read']),
    			$attr = array('title' => $alt, 'border' => '0')
    		);
		}
        
		return $view;
	}

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu à la cellule "Description" du tableau de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_description($record)
	{
        if (!empty($this -> _writer_rights['messages_edit'])) {
            return parent :: _add_contents_for_td_description($record);
        } else {
            return $this -> _get_text_for_td_description($record);
        }
	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie le texte de la cellule intitulée "Description"
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
    protected function _get_text_for_td_description($record)
    {
        return $this -> CI -> writer_messages_model -> set_doc_heading($record);      
    }

	// --------------------------------------------------------------------

	/**
	 * Renvoie l'url du lien du document dans la cellule intitulée "Description"
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Url du lien
	 */
    protected function _build_href_for_td_description($record)
    {
		$base_href = (ADMR_IS_ADMIN) ? CHEMIN.'admin/'.ADMR_BASE_HREF_PLUGIN : ADMR_BASE_HREF_PLUGIN;

        return $base_href.'&ctrl=addons/messages/add_messages&id='.$record['id'];       
    }

	// --------------------------------------------------------------------

	/**
	 * Ajoute le contenu de la cellule "Commandes" du tableau de données
     * 
	 * @access protected
	 * @param array Données de l'enregistrement
	 * @return string Contenu html de la cellule
	 */
	protected function _add_contents_for_td_commands($record)
	{
        $id = $record['id'];
       
        $view = '';
        
        // Répondre
        $view .= Html_helper :: addA(
			$href = Url_helper :: build_uri(array('to' => $record['author_id']), ADMR_BASE_HREF_PLUGIN.'&ctrl=addons/messages/add_messages'),
			$contents = Html_helper :: addImg(
				$path = ADMR_IMG_MSG,
				$alt = ucfirst($this -> _lang['answer']),
				$attr = array(
					'title' => $alt,
					'border' => '0'
				)
			)
            .'&nbsp;&nbsp;'
		);
        
        return $view;
	}

	// --------------------------------------------------------------------

    /**
	 * Vérifie la validité d'accès au document par le rédacteur
     * 
	 * @access protected
     * @return boolean
	 */
    protected function _is_doc_allowed()
    {   
        if (!empty($this -> _writer_rights[$this -> _doc_infos['name'].'_manage'])) // Autorisation d'accès à la section du document
        { 
            return TRUE;
        }
        
        return FALSE;    
    }

	// --------------------------------------------------------------------

} //End of Class